#!/usr/bin/env python
import angr
import claripy
import code

# Project
b = angr.Project('./level07')

# Make level07 arguments symbolic. We will try to resolve their concrete
# value at the end.
#
# Note: play with giving a concrete input to arg1 to see how that works.
arg1 = claripy.BVS('sym_arg', 8 * 11) # maximum 11 * 8 bits
# arg1 = "-1073741813"
arg2 = claripy.BVS('sym_arg', 8 * 44) # maximum 44 * 8 bits

# Generate an initial state: we pass arg1 and arg2 as arguments, and we
# set the maximum length for strtol to 11, to make sure the solver will
# find a solution for arg1.
st = b.factory.entry_state(args=['./level07', arg1, arg2])
st.libc.max_strtol_len = 11 # tweak

# These should in principle make the search faster. Uncomment, observe.
# st.add_constraints(arg2.chop(8)[40] == 'F')
# st.add_constraints(arg2.chop(8)[41] == 'L')
# st.add_constraints(arg2.chop(8)[42] == 'O')
# st.add_constraints(arg2.chop(8)[43] == 'W')

# Create a path group and explore it
pg = b.factory.path_group(st)
pg.explore(find=lambda p: "WIN" in p.state.posix.dumps(1))

print "pg = {}".format(pg)
if (len(pg.found) > 0):
    s = pg.found[0].state
    print "Found a solution:"
    # Find concrete values that satisfy the constraints
    print "arg1 = {} ".format(repr(s.se.any_str(arg1)))
    print "arg2 = {} ".format(repr(s.se.any_str(arg2)))
    #code.interact(local=locals())
else:
    print "Haven't found a solution, falling back to console"
    code.interact(local=locals())
